// Package ending (tenancy) is for a notification 2 months before tenancy ending
// queries property service to get list of tenancies eligible of such notification
package ending

import (
	"fmt"

	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

type Config struct {
	PropertyService     property.PropertyServiceClient
	NotificationService notification.NotificationServiceClient
}

type Handler struct {
	config Config
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid tenancy ending config")
	}

	if config.PropertyService == nil {
		return fmt.Errorf("no property service client in tenancy ending config")
	}

	if config.NotificationService == nil {
		return fmt.Errorf("no notification service client in tenancy ending config")
	}

	return nil
}

func NewHandler(config *Config) (*Handler, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	return &Handler{
		config: *config,
	}, nil
}
