// Package termination (tenancy) is for starting a claim/dispute period of 10 days after tenancy end.
// Starts BPM process for handling claim and dispute events, closing tenancy completely after 10 days or when
// all claims made in this period are resolved.
package termination

import (
	"fmt"

	"github.com/zeebe-io/zeebe/clients/go/pkg/zbc"

	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

type Config struct {
	PropertyService     property.PropertyServiceClient
	NotificationService notification.NotificationServiceClient
	ZeebeClient         zbc.Client
}

type Handler struct {
	config Config
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid tenancy termination config")
	}

	if config.PropertyService == nil {
		return fmt.Errorf("no property service client in tenancy termination config")
	}

	if config.NotificationService == nil {
		return fmt.Errorf("no notification service client in tenancy termination config")
	}

	return nil
}

func NewHandler(config *Config) (*Handler, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	return &Handler{
		config: *config,
	}, nil
}
