package model

import (
	"context"
	"time"
)

// Repository for schedulable handler invocations and recurrent tasks
type Repository interface {
	// TaskSave upserts a task for recurrent invocation
	TaskSave(ctx context.Context, task *Task) error
	// TaskList returns list of all persisted tasks
	TaskList(ctx context.Context) ([]*Task, error)
	// TaskGet returns single task by id
	TaskGet(ctx context.Context, id string) (*Task, error)
	// TaskCancel removes task from persistent storage
	TaskCancel(ctx context.Context, id string) error
	// InvocationDequeue returns next invocation for given handler prioritizing those that wasn't handled in time by
	// other instances of this service
	InvocationDequeue(ctx context.Context, handler string, retry time.Duration) (*Invocation, error)
	// InvocationAck marks invocation as successfully processed
	InvocationAck(ctx context.Context, handler string, ackid string) error
	// InvocationCancel cancels scheduled but not yet executed invocation
	InvocationCancel(ctx context.Context, handler string, id string) error
	// InvocationGet returns invocation by id
	InvocationGet(ctx context.Context, handler string, id string) (*Invocation, error)
	// InvocationEnqueue schedules handler invocation
	InvocationEnqueue(ctx context.Context, handler string, invocation *Invocation) error
}
