package model

import (
	"encoding/json"
	"time"
)

// Invocation represents single handler invocation in time
// either created standalone or as part of recurrent task
type Invocation struct {
	AckID   string    `json:"-"`              // identifier ACK to when processing invocation
	ID      string    `json:"id"`             // id of a invocation
	Instant time.Time `json:"instant"`        // instant in time to which this invocation is scheduled to
	Data    []byte    `json:"data"`           // invocation data, specific to given handler
	Retry   int       `json:"retry"`          // retry count
	Task    string    `json:"task,omitempty"` // task that originated in this invocation, if any
}

func (m *Invocation) UnmarshalBinary(data []byte) error {
	return json.Unmarshal(data, m)
}

func (m *Invocation) MarshalBinary() (data []byte, err error) {
	return json.Marshal(m)
}
