package model

import (
	"encoding/json"
	"time"
)

// Task recurrent task for producing handler invocation at constant intervals
type Task struct {
	ID      string        `json:"id"`      // task ID
	Handler string        `json:"handler"` // handler that would be used in invocation
	Active  bool          `json:"active"`  // whether this task is currently active and should recur further invocation
	Period  time.Duration `json:"period"`  // period between the invocations
	Data    []byte        `json:"data"`    // invocation data, specific to handler
	Retry   int           `json:"retry"`   // retry amount for invocations
	Armed   string        `json:"armed"`   // invocation ID this task was last armed with, if any
}

func (m *Task) UnmarshalBinary(data []byte) error {
	return json.Unmarshal(data, m)
}

func (m *Task) MarshalBinary() (data []byte, err error) {
	return json.Marshal(m)
}
